@@ -1,6 +1,6 @@
 package Biblio::Citation::Compare;
 
-use 5.010001;
+use 5.0;
 use strict;
 use warnings;
 use Text::LevenshteinXS qw(distance);
@@ -19,7 +19,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = qw( );
 
-our $VERSION = '0.03';
+our $VERSION = '0.21';
 
 # to correct bogus windows entities. unfixable ones are converted to spaces.
 my %WIN2UTF = (
@@ -57,12 +57,20 @@ my %WIN2UTF = (
     hex('9F')=> hex('0178')#  #LATIN CAPITAL LETTER Y WITH DIAERESIS
 );
 my $PARENS = '\s*([\[\(])(.+?)([\]\)])\s*';
+my $QUOTE = '"“”`¨´‘’‛“”‟„′″‴‵‶‷⁗❛❜❝❞';
+#my $TITLE_SPLIT = '(?:\?|\:|\.|!|\&quot;|[$QUOTE]\b)';
+my $TITLE_SPLIT = '(?:\?|\:|\.|!)';
 
 sub sameAuthors {
     my ($list1, $list2) = @_;
-    return 0 if $#$list1 != $#$list2;
-    for (my $i = 0; $i <= $#$list1; $i++) {
-        return 0 unless samePerson($list1->[$i],$list2->[$i]);
+    #return 0 if $#$list1 != $#$list2;
+    if ($#$list2 > $#$list1) {
+        my $t = $list1;
+        $list1 = $list2;
+        $list2 = $t;
+    }
+    for (my $i = 0; $i <= $#$list2; $i++) {
+        return 0 unless grep { samePerson($list2->[$i],$_) } @$list1;
     }
     return 1;
 }
@@ -82,6 +90,8 @@ sub sameWork {
     my $debug = 0;
 
  	my ($e, $c, $threshold,$loose,$nolinks) = @_;
+    $loose = 0 unless defined $loose;
+    $threshold = 0.15 unless $threshold;
 
     if ($debug) {
         warn "sameEntry 1: " . toString($e);
@@ -93,7 +103,6 @@ sub sameWork {
     }
 
 	return 0 if (!$c);
-    $threshold = 0.15 unless $threshold;
 
     # normalize encoding of relevant fields
     local $e->{title} = decodeHTMLEntities($e->{title});
@@ -102,25 +111,21 @@ sub sameWork {
     # first check if authors,date, and title are almost literally the same
     my $tsame = (lc $e->{title} eq lc $c->{title}) ? 1 : 0;
     my $asame = sameAuthors($e->{authors},$c->{authors});
-    my $dsame = ($e->{date} eq $c->{date}) ? 1 : 0;
-    my $firstsame = samePerson(cleanName(firstAuthor($e)),cleanName(firstAuthor($c)));
+    my $dsame = (defined $e->{date} and defined $c->{date} and $e->{date} eq $c->{date}) ? 1 : 0;
 
     if ($debug) {
         warn "tsame: $tsame";
         warn "asame: $asame";
         warn "dsame: $dsame";
-        warn "firstsame: $firstsame";
     }
 
     return 1 if ($tsame and $asame and $dsame);
 
-	my ($fname1,$lname1) = parseName(firstAuthor($e));
-	my ($fname2,$lname2) = parseName(firstAuthor($c));
-
 	# if authors quite different, not same
-    if (!$asame and my_dist_text($lname1,$lname2) / (length($lname1) + 1) > $threshold) {
+    if (!$asame) {
         #print "$lname1, $lname2<br>";
         #print my_dist_text($lname1,$lname2); 
+        warn "authors too different" if $debug;
      	return 0;
     }
 
@@ -133,7 +138,7 @@ sub sameWork {
 	return 0 if !$tsame and (
                     abs(length($e->{title}) - length($c->{title})) > 20 
                     and
-					($e->{title} !~ /:/ and $c->{title} !~ /:/)
+					($e->{title} !~ /$TITLE_SPLIT/ and $c->{title} !~ /$TITLE_SPLIT/)
                     and
 					($e->{title} !~ /$PARENS/ and $c->{title} !~ /$PARENS/)
 				); 	
@@ -149,7 +154,7 @@ sub sameWork {
 
     # check dates
     my $compat_dates = $dsame;
-    if (!$dsame and $e->{date} =~ /^\d\d\d\d$/ and $c->{date} =~ /^\d\d\d\d$/ ) {
+    if (!$dsame and defined $e->{date} and defined $c->{date} and $e->{date} =~ /^\d\d\d\d$/ and $c->{date} =~ /^\d\d\d\d$/ ) {
 
         $compat_dates = 0;
         #disabled for most cases because we want to conflate editions and republications for now. 
@@ -165,7 +170,7 @@ sub sameWork {
     } 
     
    # authors same, loosen for title 
-    if (($asame or $firstsame) and $compat_dates) {
+    if ($asame and $compat_dates) {
        $loose = 1;
     }
 
@@ -193,16 +198,16 @@ sub sameWork {
     #print $score . "<br>\n";
  	return 1 if ( $score < $threshold);
 
-	# now if loose mode and only one of the titles has a ":", compare the part before ":" with the other title instead
+	# now if loose mode and only one of the titles has a ":" or other punctuation, compare the part before the punc with the other title instead
     if ($loose) {
 
         warn "loose: $str1 -- $str2" if $debug;
         return 1 if (my_dist_text($str1,$str2) / (length($str1) +1) < $threshold);
 
-        if ($e->{title} =~ /(.+):(.+)/) {
+        if ($e->{title} =~ /(.+)\s*$TITLE_SPLIT\s*(.+)/) {
 
             my $str1 = _strip_non_word($1);
-            if ($c->{title} =~ /(.+):(.+)/) {
+            if ($c->{title} =~ /(.+)\s*$TITLE_SPLIT\s*(.+)/) {
                 return 0;
             } else {
                 if (my_dist_text($str1,$str2) / (length($str1) +1)< $threshold) {
@@ -210,7 +215,7 @@ sub sameWork {
                 }
             }
 
-        } elsif ($c->{title} =~ /(.+):(.+)/) {
+        } elsif ($c->{title} =~ /(.+)\s*$TITLE_SPLIT\s*(.+)/) {
 
             my $str2 = _strip_non_word($1);
             if (my_dist_text($str1,$str2) / (length($str1) +1)< $threshold) {
@@ -286,7 +291,7 @@ sub safe_decode {
 
 sub toString {
     my $h = shift;
-    print join("; ",@{$h->{authors}}) . " ($h->{date}) $h->{title}\n";
+    return join("; ",@{$h->{authors}}) . " ($h->{date}) $h->{title}\n";
 }
 
 1;
@@ -298,7 +303,7 @@ Biblio::Citation::Compare - Perl extension for performing fuzzy comparisons betw
 
 =head1 SYNOPSIS
 
-  use Biblio::Citation::Compare 'sameWork';
+  use Biblio::Citation::Compare 'sameWork','sameAuthors';
 
   sameWork(
     # first item
@@ -309,16 +314,22 @@ Biblio::Citation::Compare - Perl extension for performing fuzzy comparisons betw
     },
     # second item
     {
-        authors => ['Bourget, David J. R.','Lukasiak, Zbigniew'
+        authors => ['Bourget, David J. R.','Lukasiak, Zbigniew'],
         title => "A paper with such nd such a tlitle",
         date => undef
     }
-   );
-
-   # true!
-
   );
 
+  # true!
+
+  sameAuthors(
+    ['Dave Bourget','Z Lukasiak'],
+    ['Bourget DJR','Zbigniew Z. Lukasiak']
+  );
+
+  # true!
+
+
 =head1 DESCRIPTION
 
 This module exports two subroutines which perform fuzzy comparisons between citations (described using Perl hashes) and author lists. The subroutine attempt to determine if the citations and author lists refer to the same works and ordered lists of authors, respectively.
@@ -327,13 +338,13 @@ This module exports two subroutines which perform fuzzy comparisons between cita
 
 =head2 sameWork(hashref citation1, hashref citation2): boolean
 
-Takes as input two citations in a simple format illustrated in the synopsis. Returns true iff the two citations plausibly refer to the same work. A number of factors are taken into account to make the evaluation resistant to random variations. Among them: names are normalized and compared fuzzily using L<Text::Names>, allowances are made for random typos, allowances are made for short and long versions of titles (esp with titles containing a colon), small but important variations as in "Paper title part 1" and "Paper title part 2" are taken into account. The algorithm has been use to merge multiple data sources on L<PhilPapers.org>. 
+Takes as input two citations in a simple format illustrated in the synopsis. Returns true iff the two citations plausibly refer to the same work. A number of factors are taken into account to make the evaluation resistant to random variations. Among them: names are normalized and compared fuzzily using L<Text::Names>, allowances are made for random typos, allowances are made for short and long versions of titles (esp with titles containing a colon), small but important variations as in "Paper title part 1" and "Paper title part 2" are taken into account. The algorithm has been used to merge multiple data sources on L<PhilPapers.org>. 
 
-Some advanced additional parameters are not explained here; they can only be explained by pointing to the source code. There use should not normally be necessary.
+Some advanced additional parameters are not explained here; they can only be explained by pointing to the source code. Their use should not normally be necessary.
 
 =head2 sameAuthors(arrayref list1, arrayref list2): boolean
 
-Returns true if the two list are plausibly lists of the same authors. This is merely a convenient wrapper over L<Text::Names>::samePerson.
+Returns true if the two lists are plausibly lists of the same authors. This is merely a convenient wrapper over L<Text::Names>::samePerson.
 
 =head2 EXPORT
 
